import React from 'react';
import E from 'wangeditor';
import { fetch, apis } from '../../utils/request';

/**
 * @param {*} props
 * @returns
 */
class Wangedit extends React.Component {
  state = {
    html: undefined,
    contenteditable: this.props.contenteditable == false ? false : true
  }
  editorRef = React.createRef()
  editor = undefined;

  render() {
    return (
      <div ref={this.editorRef} style={{ textAlign: 'left' }}>
      </div>
    );
  }

  componentDidUpdate(preProps) {
    if (preProps.value != this.props.value) {
      if (this.editor) {
        this.editor.txt.html(this.props.value)
      }
    }
  }

  componentDidMount() {
    const elem = this.editorRef.current
    this.editor = new E(elem)
    // 使用 onchange 函数监听内容的变化
    this.editor.config.onchange = html => {
      this.setState({ html })

      if (typeof this.props.onChange === 'function') {
        this.props.onChange(html)
      }
    }
    this.editor.config.zIndex = 100
    this.editor.config.uploadImgMaxLength = 1
    this.editor.config.customUploadImg = function (files, insert) {
      if (!files.length) return
      
      const data = new FormData()
      data.append('file', files[0])

      fetch(apis.image.upload)({data}).then(insert)
    }
    this.editor.config.menus = [
      'head',  // 标题
      'bold',  // 粗体
      'fontSize',  // 字号
      'fontName',  // 字体
      'italic',  // 斜体
      'underline',  // 下划线
      'strikeThrough',  // 删除线
      'foreColor',  // 文字颜色
      'backColor',  // 背景颜色
      'list',  // 列表
      'justify',  // 对齐方式
      'quote',  // 引用
      'image',  // 插入图片
      'undo',  // 撤销
      'redo'  // 重复
    ]
    
    // 过滤 word 字符
    this.editor.config.pasteFilterStyle = false
    this.editor.config.pasteTextHandle = function(content) {
      const regs = [
        /<!--\[if [\s\S]*?endif\]-->/ig,
        /<[a-zA-Z0-9]+\:[^>]+>[^>]*<\/[a-zA-Z0-9]+\:[^>]+>/ig,
        /<[a-zA-Z0-9]+\:[^>]+\/>/ig,
        /<style>[\s\S]*?<\/style>/ig,
        new RegExp('\u2029', 'ig'),     // 替换word分隔符 序号 8233
      ]

      return regs.reduce((acc, reg) => {
        return acc.replace(reg, '')
      }, content)
    }

    this.editor.create()
    this.editor.$textElem.attr('contenteditable',this.state.contenteditable);
    this.editor.config.uploadImgShowBase64 = true
    this.editor.txt.html(this.props.value)
  }

  componentDidUpdate(props, state) {
    if (this.props.value && !state.html) {
      if (this.editor) {
        this.editor.txt.html(this.props.value)
      }
    }
  }

  /**
   *增加这个 shouldComponentUpdate 生命函数
    处理自动聚焦到富文本上
   *
   * @param {*} nextProps
   * @returns
   * @memberof Wangedit
   */
  shouldComponentUpdate(nextProps) {
    return nextProps.value !== this.editor.txt.html()
  }
}

export default Wangedit

